import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Hashtable;

/**
 * Database handler interface
 * @author metax
 */
public abstract class Database {
	
	private Hashtable<String, PreparedStatement> statements;
	protected Connection conn;
	
	/**
	 * Initialize the statements.
	 * @throws SQLException
	 */
	protected void initStatements() throws SQLException {
		if (conn == null)
			throw new IllegalStateException("Connection not established, yet.");
		this.statements = new Hashtable<String, PreparedStatement>();
	}
	
	/**
	 * Inititalize the database.
	 * @throws ClassNotFoundException if the Database driver class is not available
	 * @throws SQLException if the connection of any prepared statement fail
	 */
	public abstract void connect() throws ClassNotFoundException, SQLException;
	
	/**
	 * Stop the database service.
	 */
	public abstract void destroy() throws SQLException;
	
	/**
	 * Prepare a statement.
	 * @param key the key to identify the statement.
	 * @param sql the sql for the statement
	 * @throws SQLException if the query encounters an error
	 */
	public void prepare(String key, String sql) throws SQLException {
		this.statements.put(key, conn.prepareStatement(sql));
	}
	
	/**
	 * Check, if a named PreparedStatement was defined.
	 * @param key the key of the statement
	 * @return true, if the statement exists
	 */
	public boolean isPrepared(String key) {
		try {
			return statements.containsKey(key) && !statements.get(key).isClosed();
		} catch (SQLException e) {
			return false;
		}
	}
	
	/**
	 * Get the prepared statement identified by key.
	 * If there is no such statement, null is returned.
	 * @param key the key
	 * @return the statement
	 */
	public PreparedStatement get(String key) {
		return statements.get(key);
	}
	
	/**
	 * Get the prepared statement identified by key and prepare it, if it wasn't prepared before.
	 * @param key the key
	 * @param sql the sql to prepare the statement
	 * @return statement
	 * @throws SQLException if {@link #prepare(String, String)} throws a SQLException
	 */
	public PreparedStatement getAndPrepare(String key, String sql) throws SQLException {
		try {
			if (conn == null || conn.isClosed() || !conn.isValid(1)) {
				System.out.println("[DEBUG] Database-Connection seems to be closed. Trying to reopen it ...");
				connect();
			}
		} catch (ClassNotFoundException e) {
			throw new SQLException(e);
		}
		if (!isPrepared(key)) {
			prepare(key, sql);
		}
		return get(key);
	}

}
