import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Set;
import java.util.TreeSet;

public class MyCache extends AbstractCache {

	// Keys fuer Prepared Statements
	private static final String KEY_READDATA = "MyCache_Read";
	private static final String KEY_INSERTDATA = "MyCache_Insert";
	private static final String KEY_UPDATEDATA = "MyCache_Update";
	private static final String KEY_DELETEDATA = "MyCache_Delete";
	
	// SQL Queries
	private static final String QUERY_READDATA =
		"select id, value from data";
	private static final String QUERY_INSERTDATA =
		"insert into data (id, value) values (?, ?)";
	private static final String QUERY_UPDATEDATA =
		"update data set value = ? where id = ? limit 1";
	private static final String QUERY_DELETEDATA =
		"delete from data where id = ? limit 1";
	
	// Zwischengespeicherte Daten
	private final Hashtable<Integer, String> data;
	private final TreeSet<String> values;
	
	public MyCache(Database db) throws SQLException {
		super(db);
		this.data = new Hashtable<Integer, String>();
		this.values = new TreeSet<String>();
		reload();
	}
	
	/**
	 * (Neues) Einlesen.
	 * @throws SQLException bei SQL Fehler
	 */
	@Override
	public void reload() throws SQLException {
		writeLock();
		try {
			PreparedStatement stmt = db().getAndPrepare(KEY_READDATA, QUERY_READDATA);
			stmt.execute();
			ResultSet res = stmt.getResultSet();
			
			// Leeren der aktuellen Daten
			this.data.clear();
			this.values.clear();
			
			// Neues Einlesen
			while (res.next()) {
				int id = res.getInt("id");
				String value = res.getString("value");
				this.values.add(value);
				this.data.put(id, value);
			}
		} finally {
			writeUnlock();
		}
	}
	
	/**
	 * Wert zu einer ID zurückliefern.
	 */
	public String getValue(int id) {
		readLock();
		try {
			return this.data.get(id);
		} finally {
			readUnlock();
		}
	}
	
	/**
	 * Liste alle verfügbarer IDs zurückliefern.
	 */
	public Set<Integer> getIDs() {
		readLock();
		try {
			Set<Integer> result = new TreeSet<Integer>(this.data.keySet());
			return Collections.unmodifiableSet(result); 
		} finally {
			readUnlock();
		}
	}
	
	/**
	 * Liste alle verfügbarer Werte zurückliefern.
	 */
	public Set<String> getValues() {
		readLock();
		try {
			Set<String> result = new TreeSet<String>(this.values);
			return Collections.unmodifiableSet(result);
		} finally {
			readUnlock();
		}
	}
	
	/**
	 * Wertepaar einfügen.
	 */
	public void insert(int id, String value) throws SQLException {
		writeLock();
		try {
			PreparedStatement stmt = db().getAndPrepare(KEY_INSERTDATA, QUERY_INSERTDATA);
			stmt.setInt(1, id); // Parameter 1
			stmt.setString(2, value); // Parameter 2
			stmt.execute();
			
			// lokale Änderung
			this.data.put(id, value);
			this.values.add(value);
		} finally {
			writeUnlock();
		}
	}
	
	/**
	 * Wert neu zuordnen.
	 */
	public void update(int id, String value) throws SQLException {
		writeLock();
		try {
			PreparedStatement stmt = db().getAndPrepare(KEY_UPDATEDATA, QUERY_UPDATEDATA);
			stmt.setString(1, value); // Parameter 1
			stmt.setInt(2, id); // Parameter 2
			stmt.execute();
			
			// lokale Änderung
			this.data.put(id, value);
			this.values.clear(); // Values neu aufarbeiten
			for (String val : this.data.values()) {
				this.values.add(val);
			}
		} finally {
			writeUnlock();
		}
	}
	
	/**
	 * Wertepaar löschen.
	 */
	public void delete(int id) throws SQLException {
		writeLock();
		try {
			PreparedStatement stmt = db().getAndPrepare(KEY_DELETEDATA, QUERY_DELETEDATA);
			stmt.setInt(1, id); // Parameter 1
			stmt.execute();
			
			// lokale Änderung
			this.data.remove(id);
			this.values.clear(); // Values neu aufarbeiten
			for (String val : this.data.values()) {
				this.values.add(val);
			}
		} finally {
			writeUnlock();
		}
	}
	
}
